@@ -30,8 +30,8 @@ def files():
 @click.option('--service_account',
               default=None,
               help='The path to the service account key json file')
+@click.option('--modality', default=None, help='The modality of the file')
 @click.argument('client_id')
-@click.argument('modality')
 @click.argument('mrn')
 @click.argument('metadata')
 @click.argument('file_path')
@@ -41,11 +41,6 @@ def upload_file(client_id, modality, mrn, metadata, file_path, service_account):
     Arguments: 
         
         CLIENT_ID:      The specific UUID4 for a client.
-        MODALITY:       The files MODALITY. acceptable parameters are:
-            HVF
-            OCT_RNFL
-            OCT_RETINA
-            AVS
         MRN:            The Patients medical record number
         METADATA:       The json string or json file of the file you wish to upload.
                         The structure is:
@@ -63,7 +58,10 @@ def upload_file(client_id, modality, mrn, metadata, file_path, service_account):
     """
     file_client = ClientOps.Client(client_id,
                                    credentials_file_path=service_account)
-    result = file_client.upload_file(modality, mrn, metadata, file_path)
+    result = file_client.upload_file(mrn,
+                                     metadata,
+                                     file_path,
+                                     modality=modality)
     sys.exit(int(not result))
 
 
