@@ -46,7 +46,7 @@ class Client:
         self.__argument_validator = Argument.Validator()
         self.__document_classifier = DocumentClassifierApi.DocClassifierClient()
 
-    def upload_file(self, modality, mrn, metadata, file_path):
+    def upload_file(self, mrn, metadata, file_path, modality=None):
         '''
             Top level function that uploads a file to a Datalake, This will aslo add the 
             Patient data to the PATIENTS table if applicable
@@ -61,11 +61,14 @@ class Client:
         # validate arguments
         arguments = {
             'client_id': self.client_uuid,
-            'modality': modality,
             'mrn': mrn,
             'file_metadata': metadata,
             'file_path': file_path
         }
+
+        if modality is not None:
+            arguments['modality'] = modality
+
         data_lake_command = 'file'
 
         Properties.Properties.instance()
@@ -239,8 +242,13 @@ class Client:
         # run the document classifier, check if modality matches predicted modality
         classified_modality = self.__document_classifier_operations(
             arguments['mrn'], arguments['file_path'])
+
+        if 'modality' not in arguments:
+            arguments['modality'] = classified_modality
+
         if not classified_modality or classified_modality is None:
             classified_modality = 'Failed To Classify'
+            arguments['modality'] = 'UNKNOWN'
             warnings.warn("Request to document classifier failed.")
             self.__client_ops_logger.log_text(
                 'failed to run document classifier on file ' +
@@ -253,6 +261,13 @@ class Client:
 
         file_metadata['CLASSIFIED_MODALITY'] = classified_modality
 
+        if 'MODALITY' not in file_metadata:
+            file_metadata['MODALITY'] = arguments['modality']
+
+        if 'FILE_PATH' not in file_metadata:
+            file_metadata['FILE_PATH'] = str(file_metadata['MODALITY']) + \
+            '/' + str(file_metadata['PATIENT_ID'])
+
         # run command
         return Upload.Client(self.client_uuid).upload_file_to_datalake(
             arguments['modality'], arguments['mrn'], file_metadata,
