@@ -4,9 +4,9 @@
 #
 
 import re
-import sys
 import cv2
 import os
+
 from indxdatalaketools.GoogleClients.GoogleClients import GcpClients
 
 import numpy as np
@@ -16,6 +16,8 @@ from indxdatalaketools.ClientTools.PatientsTable import Insert
 from indxdatalaketools.ClientTools.Changelog import Insert as ChnglogInsert
 from indxdatalaketools.DataStandardizer import PatientTable
 
+from indxdatalaketools import DataLakeGlobals
+
 
 class Client:
     '''
@@ -59,6 +61,8 @@ class Client:
         file_contents = self.__read_file_contents(file_path)
         file_extension = os.path.splitext(file_path)[1]
 
+        DataLakeGlobals.SIZE_OF_FILES += len(file_contents) * .000001
+
         # convert bmp to png
         if '.bmp' == file_extension.lower():
             file_contents = self.__convert_bmp_to_png(file_contents)
