@@ -3,19 +3,15 @@
 #   Created on 3/16/2022
 #
 
-import sys
-import time
-import google.cloud.logging
-import logging
 from google.cloud import bigquery
 from google.api_core.exceptions import NotFound
+from google.api_core.exceptions import ServiceUnavailable
 from indxdatalaketools import Helpers
 from indxdatalaketools.DataLakeGlobals import DATASET_OWNER
 from indxdatalaketools.GoogleClients.GoogleClients import GcpClients
 from indxdatalaketools.Helpers import print_error
-from google.cloud.bigquery_storage_v1 import types
-from google.cloud.bigquery_storage_v1 import writer
 from google.cloud.bigquery.enums import EntityTypes
+from indxdatalaketools.GoogleClientWrappers.retry_decorator import retry_decorator
 
 
 class Wrapper:
@@ -33,58 +29,30 @@ class Wrapper:
         self.__google_big_query_client = GcpClients.instance(
         ).get_bigquery_client()
 
-    def insert_to_bq_table(self, bq_storage_client, serialized_row,
-                           request_template, stream_name, parent):
+    @retry_decorator()
+    def insert_to_bq_table(self, patient_data, table_name):
         '''
             Function that inserts data into a big query table
             Args:
-                bq_client (google.cloud.bigquery.client): The client that interacts
-                    with bigquery api
-                serialized_row: The serialized proto buffer row
-                request_template: The template the request needs to follow
-                stream_name (string): The name of the stream
-                parent: The parent
+                patient_data (list): a list of dictionaries containing  patient data
+                table_name (string): the name of the table we want to insert to
             Returns:
-                repsonse: The response from the API call
+                boolean: True if the data was inserted, False if otherwise
         '''
-        append_rows_stream = writer.AppendRowsStream(bq_storage_client,
-                                                     request_template)
-        proto_rows = types.ProtoRows()
-        proto_rows.serialized_rows.append(serialized_row)
-
-        # Set an offset to allow resuming this stream if the connection breaks.
-        # Keep track of which requests the server has acknowledged and resume the
-        # stream at the first non-acknowledged message. If the server has already
-        # processed a message with that offset, it will return an ALREADY_EXISTS
-        # error, which can be safely ignored.
-        #
-        # The first request must always have an offset of 0.
-        request = types.AppendRowsRequest()
-        request.offset = 0
-        proto_data = types.AppendRowsRequest.ProtoData()
-        proto_data.rows = proto_rows
-        request.proto_rows = proto_data
-        request.write_stream = stream_name
-
         try:
-            response_future_1 = append_rows_stream.send(request)
+            errors = GcpClients.instance().get_bigquery_client(
+            ).insert_rows_json(table_name, patient_data)
+            if errors == []:
+                return True
+            print_error(errors)
+            return False
+        except ServiceUnavailable:
+            raise ServiceUnavailable('503 retry')
         except Exception as e:
-            print("ERROR:" + str(e))
-            return None
-
-        append_rows_stream.close()
-
-        bq_storage_client.finalize_write_stream(name=stream_name)
-
-        batch_commit_write_streams_request = types.BatchCommitWriteStreamsRequest(
-        )
-        batch_commit_write_streams_request.parent = parent
-        batch_commit_write_streams_request.write_streams = [stream_name]
-        bq_storage_client.batch_commit_write_streams(
-            batch_commit_write_streams_request)
-
-        return response_future_1
+            print_error(str(e))
+            return False
 
+    @retry_decorator()
     def batch_load_from_gcs(self, gcs_uri, table_destination, file_format):
         '''
             Batch loads a file from gcs into big query
@@ -100,12 +68,14 @@ class Wrapper:
             use_avro_logical_types=True,
             write_disposition=bigquery.WriteDisposition.WRITE_TRUNCATE,
             source_format=self.file_format_map[file_format])
-        load_job = self.__google_big_query_client.load_table_from_uri(
-            gcs_uri, table_destination,
-            job_config=job_config)  # Make an API request.
         try:
+            load_job = self.__google_big_query_client.load_table_from_uri(
+                gcs_uri, table_destination,
+                job_config=job_config)  # Make an API request.
             load_job.result()
             return True
+        except ServiceUnavailable as e:
+            raise ServiceUnavailable('503 retry')
         except Exception as e:
             print_error(str(e))
             return False
@@ -132,10 +102,10 @@ class Wrapper:
             return True
 
         print("Creating dataset: " + dataset_id)
-        self.create_dataset(dataset)
 
-        return self.check_if_dataset_exists(dataset_id)
+        return self.create_dataset(dataset)
 
+    @retry_decorator()
     def get_dataset(self, dataset_id):
         '''
             Gets the dataset in bigquery
@@ -146,24 +116,45 @@ class Wrapper:
         '''
         try:
             return bigquery.Dataset(dataset_id)
+        except ServiceUnavailable as e:
+            raise ServiceUnavailable('503 retry')
         except Exception as e:
             Helpers.print_error('Could not retreive dataset ' + dataset_id +
                                 " " + str(e))
             return None
 
-    def create_table_if_not_exist(self, dataset_id, table_name, schema):
-        '''
-            Creates the client's PATIENT table if it does not already exist
-            Args:
-                dataset_id (string): The id of the dataset for the table
-                table_name (string): The name of the table we wish to create
-            Returns:
-                boolean: True if the table was created or already existed, false
-                    if table creation failed
+    def create_table_if_not_exist(self,
+                                  dataset_id,
+                                  table_name,
+                                  schema,
+                                  partition=None,
+                                  clustering=None):
+        '''Creates the client's PATIENT table if it does not already exist
+
+        This function also takes in optional partitions and clustering for the table to be
+        created
+
+        Args:
+            dataset_id (string): The id of the dataset for the table
+            table_name (string): The name of the table we wish to create
+            schema (list): a list of bigquery.SchemaField representing the table schema
+        Returns:
+            boolean: True if the table was created or already existed, false
+                if table creation failed
         '''
         # create table object
         table_id = dataset_id + "." + table_name
         table = bigquery.Table(table_id, schema=schema)
+        if partition is not None:
+            table.range_partitioning = bigquery.RangePartitioning(
+                # To use integer range partitioning, select a top-level REQUIRED /
+                # NULLABLE column with INTEGER / INT64 data type.
+                field=partition,
+                range_=bigquery.PartitionRange(start=0, end=4000, interval=1),
+            )
+
+        if clustering is not None:
+            table.clustering_fields = clustering
 
         if self.check_if_table_exists(table_id):
             print(
@@ -171,10 +162,9 @@ class Wrapper:
             return True
 
         print("Creating table " + table_id)
-        table = self.create_table(table)
-
-        return self.check_if_table_exists(table_id)
+        return self.create_table(table)
 
+    @retry_decorator()
     def create_table(self, table):
         '''
             Creates Big Query Table
@@ -184,11 +174,15 @@ class Wrapper:
                 bigquery.Table: The created Table
         '''
         try:
-            return self.__google_big_query_client.create_table(table)
+            self.__google_big_query_client.create_table(table)
+            return True
+        except ServiceUnavailable as e:
+            raise ServiceUnavailable('503 retry')
         except Exception as e:
-            Helpers.print_error("Could not connect to GCS client " + str(e))
-            sys.exit(1)
+            Helpers.print_error("ERROR in creating table: " + str(e))
+            return False
 
+    @retry_decorator()
     def create_dataset(self, dataset):
         '''
             Creates Big Query Dataset
@@ -199,15 +193,20 @@ class Wrapper:
         '''
         try:
             self.__google_big_query_client.create_dataset(dataset, timeout=30)
+            return True
+        except ServiceUnavailable as e:
+            raise ServiceUnavailable('503 retry')
         except Exception as e:
             Helpers.print_error("Could not connect to GCS client " + str(e))
-            sys.exit(1)
+            return False
 
+    @retry_decorator()
     def check_if_table_exists(self, table_id):
         '''
             Checks if the table Id passed exists on GCP
             Args:
                 dataset_id (string): The table {PROJECT_ID}.{CLIENT_UUID}.PATIENTS
+                retries (int): Optional parameter for the number of retries this function will do
             Returns:
                 boolean: True if the table exists, false if other wise
         '''
@@ -215,12 +214,15 @@ class Wrapper:
             self.__google_big_query_client.get_table(
                 table_id)  # Make an API request.
             return True
+        except ServiceUnavailable as e:
+            raise ServiceUnavailable('503 retry')
         except NotFound:
             return False
         except Exception as e:
             print(str(e))
             return False
 
+    @retry_decorator()
     def check_if_dataset_exists(self, dataset_id):
         '''
             Checks if the dataset ID passed exists on GCP
@@ -233,12 +235,15 @@ class Wrapper:
             self.__google_big_query_client.get_dataset(
                 dataset_id)  # Make an API request.
             return True
+        except ServiceUnavailable as e:
+            raise ServiceUnavailable('503 retry')
         except NotFound:
             return False
         except Exception as e:
             print(str(e))
             return False
 
+    @retry_decorator()
     def delete_table(self, table_id):
         '''
             Deletes a table
@@ -250,22 +255,32 @@ class Wrapper:
         try:
             self.__google_big_query_client.delete_table(table_id)
             return True
+        except ServiceUnavailable as e:
+            raise ServiceUnavailable('503 retry')
         except Exception as e:
             print_error('Failed to Delete table: ' + table_id + " " + str(e))
             return False
 
-    def perform_query(self, query):
-        '''
-            Performs a big query query
-            Args:
-                query (string): the query we wish to perform
-            Returns:
-                query_job: The job of the query 
+    @retry_decorator()
+    def perform_query(self, query, job_config=None):
+        '''Performs a bigquery query
+
+        An optional job config can be passed, usually used when querying clustered data
+
+        Args:
+            query (string): the query we wish to perform
+            job_config(google.cloud.bigquery.JobConfig): an optional job configuration for
+                the query
+        Returns:
+            query_job: The job of the query 
         '''
         try:
-            query_job = self.__google_big_query_client.query(query)
+            query_job = self.__google_big_query_client.query(
+                query, job_config=job_config)
             rows = query_job.result()
             return rows
+        except ServiceUnavailable as e:
+            raise ServiceUnavailable('503 retry')
         except Exception as e:
             Helpers.print_error('Could not connect to GCS Big Query ' + str(e))
             return None
@@ -301,6 +316,7 @@ class Wrapper:
 
         return self.update_dataset(dataset, ["access_entries"])
 
+    @retry_decorator()
     def update_dataset(self, dataset, fields):
         '''
             Updates a dataset with the fields passed
@@ -313,6 +329,8 @@ class Wrapper:
         try:
             self.__google_big_query_client.update_dataset(dataset, fields)
             return True
+        except ServiceUnavailable as e:
+            raise ServiceUnavailable('503 retry')
         except Exception as e:
             print_error(e)
             return False
