@@ -4,7 +4,9 @@
 #
 
 from indxdatalaketools.GoogleClients.GoogleClients import GcpClients
+from google.api_core.exceptions import ServiceUnavailable
 from indxdatalaketools.Helpers import print_error
+from indxdatalaketools.GoogleClientWrappers.retry_decorator import retry_decorator
 
 
 class Wrapper:
@@ -14,8 +16,10 @@ class Wrapper:
     __google_cloud_tasks_client = None
 
     def __init__(self):
-        self.__google_cloud_tasks_client = GcpClients.instance().get_tasks_v2_client()
+        self.__google_cloud_tasks_client = GcpClients.instance(
+        ).get_tasks_v2_client()
 
+    @retry_decorator()
     def create_http_task(self, parent, task):
         '''
             Function that creates an http task
@@ -31,6 +35,8 @@ class Wrapper:
                 create_task(request={"parent": parent, "task": task})
             print("Created task {}".format(response.name))
             return True
+        except ServiceUnavailable as e:
+            raise ServiceUnavailable('503 retry')
         except Exception as e:
             print_error(e)
             return False
\ No newline at end of file
