@@ -4,7 +4,9 @@
 #
 
 from google.api_core.exceptions import NotFound
+from google.api_core.exceptions import ServiceUnavailable
 from indxdatalaketools import Helpers
+from indxdatalaketools.GoogleClientWrappers.retry_decorator import retry_decorator
 from indxdatalaketools.GoogleClients.GoogleClients import GcpClients
 
 
@@ -16,6 +18,7 @@ class Wrapper:
         self.__scheduler_client = GcpClients.instance().get_scheduler_v1_client(
         )
 
+    @retry_decorator()
     def check_job_exists(self, job):
         '''
             Chekcs if the job already exists
@@ -27,9 +30,12 @@ class Wrapper:
         try:
             self.__scheduler_client.get_job(request=job)
             return True
+        except ServiceUnavailable:
+            raise ServiceUnavailable('503 retry')
         except NotFound:
             return False
 
+    @retry_decorator()
     def try_create_job(self, parent, job):
         ''' 
             Tries to create a cloud scheduler job
@@ -46,6 +52,8 @@ class Wrapper:
             })
             print('Scheduled job was created')
             return True
+        except ServiceUnavailable:
+            raise ServiceUnavailable('503 retry')
         except Exception as exception:
             Helpers.print_error('Could not create cloud scheduler job: ' +
                                 str(exception))
